# Generated by Django 3.1.3 on 2020-12-08 17:53

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_ab_testing', '0005_hypothesis_and_created_by'),
    ]

    operations = [
        migrations.AlterField(
            model_name='abtest',
            name='sample_size',
            field=models.PositiveIntegerField(validators=[django.core.validators.MinValueValidator(1)]),
        ),
    ]
