# Generated by Django 4.1.6 on 2023-03-02 13:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0078_referenceindex'),
        ('wagtail_ab_testing', '0011_rename_treatment_to_variant_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='abtest',
            name='variant_revision',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.revision'),
        ),
    ]
