# Generated by Django 3.1.3 on 2020-11-05 18:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_ab_testing', '0002_abtesthourlylog'),
    ]

    operations = [
        migrations.AddField(
            model_name='abtest',
            name='winning_variant',
            field=models.CharField(choices=[('control', 'Control'), ('treatment', 'Treatment')], max_length=9, null=True),
        ),
    ]
