{% extends "wagtailadmin/base.html" %}
{% load wagtailadmin_tags i18n %}
{% block titletag %}{% trans "Create an A/B Test" %} - {{ page.title }}{% endblock %}

{% block content %}
    {% trans "Create an A/B Test" as title %}
    {% include "wagtailadmin/shared/header.html" with title=title subtitle=page.title icon='people-arrows' merged=1 %}

    <div class="nice-padding" style="padding-top: 20px;">
        <p>{% trans "This will create an A/B test comparing the published version of this page and the draft with the following differences:" %}</p>

        {% if differences %}
            {% include "./includes/comparison.html" %}

            <a class="button button-primary" href="{% url 'wagtail_ab_testing_admin:add_ab_test_form' page.id %}">{% trans "Create this test" %}</a>
        {% else %}
            <div class="help-block help-warning">
                <p>{% trans "There are no differences, please go back and make some changes to set up a new test" %}
            </div>
        {% endif %}
    </div>
{% endblock %}
