{% extends "./results.html" %}
{% load wagtailcore_tags wagtailadmin_tags i18n %}
{% block titletag %}{% trans "A/B testing activity log" %} - {{ page.title }} - {{ ab_test.name }}{% endblock %}

{% block content %}
    {{ block.super }}

    {% if action_menu.default_item %}
        <form action="{% url 'wagtailadmin_pages:edit' page.id %}" method="POST">
            {% csrf_token %}

            <footer class="footer">
                <nav aria-label="{% trans 'Actions' %}">
                    <ul>
                        <li class="actions actions--primary footer__container">
                            <div class="dropdown dropup dropdown-button match-width">
                                {{ action_menu.render_html }}
                            </div>
                        </li>
                    </ul>
                </nav>
            </footer>
        </form>
    {% endif %}
{% endblock %}
