# Generated by Django 3.1.3 on 2020-12-09 11:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_ab_testing', '0007_grant_moderators_add_abtest_permission'),
    ]

    operations = [
        migrations.AlterField(
            model_name='abtest',
            name='status',
            field=models.CharField(choices=[('draft', 'Draft'), ('running', 'Running'), ('paused', 'Paused'), ('cancelled', 'Cancelled'), ('finished', 'Finished'), ('completed', 'Completed')], default='draft', max_length=20),
        ),
    ]
